@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2010 Joshua D Bartlett
+# Copyright (C) 2006-2011 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,13 +17,18 @@
 
 from collections import defaultdict
 import string
+import logging
 
 from trosnoth.model.universe import Abort
+from trosnoth.model.upgrades import allUpgrades
 from trosnoth.model.zone import ZoneState
 from trosnoth.messages import (AchievementUnlockedMsg, TaggingZoneMsg,
-        PlayerKilledMsg, PlayerHasUpgradeMsg, ShotFiredMsg)
+        PlayerKilledMsg, PlayerHasUpgradeMsg, ShotFiredMsg, DeleteUpgradeMsg,
+        ChatMsg, ShotAbsorbedMsg, RespawnMsg, RemovePlayerMsg, GameOverMsg)
 from trosnoth.utils.event import Event
 
+log = logging.getLogger('achievementlist')
+
 ###########
 # Helpers #
 ###########
@@ -44,6 +49,9 @@ class Achievement(object):
     keepProgress = False
     afterGame = False
 
+    def rejoined(self, player):
+        pass
+
     def processMessage(self, msg):
         msgType = type(msg).__name__
         methodname = 'got%s' % (msgType,)
@@ -54,7 +62,7 @@ class Achievement(object):
     def _sendUnlocked(self, player):
         self.outPlug.send(AchievementUnlockedMsg(player.id,
                 self.idstring.encode()))
-        print self.achievementString(player)
+        log.debug(self.achievementString(player))
 
     def achievementString(self, player):
         if self.name == "":
@@ -147,40 +155,41 @@ class NoLimit(PlayerAchievement):
         return True
 
 class OnceEverPerPlayer(PlayerAchievement):
+    '''
+    self.unlocked is "unlocked this time".
+    self.previouslyUnlocked is "unlocked previously".
+    '''
+    keepProgress = True
     def __init__(self, player, world, outPlug):
         super(OnceEverPerPlayer, self).__init__(player, world, outPlug)
-        # Unlocked is "unlocked this time". Ignore is "unlocked previously".
-        # Just in case we want to distinguish between them.
         self.unlocked = False
-        self.ignore = False
-        self.keepProgress = True
+        self.previouslyUnlocked = False
         if self.player is not None:
             self.readExistingData()
 
     def readExistingData(self):
-        if self.player.user is not None:
-            if self.player.user.achievements.has_key(self.idstring):
-                self.ignore = self.player.user.achievements[self.idstring].get(
-                        'unlocked', False)
-                self.progress = self.player.user.achievements[
-                        self.idstring].get('progress', 0)
-            else:
-                self.ignore = False
-                self.progress = 0
+        user = self.player.user
+        if user is None:
+            self.previouslyUnlocked = False
+            self.progress = 0
         else:
-            self.ignore = True
+            userAchievements = user.achievements.get(self.idstring, {})
+            self.previouslyUnlocked = userAchievements.get('unlocked', False)
+            self.progress = userAchievements.get('progress', 0)
 
     def achievementUnlocked(self):
-        if not self.unlocked and not self.ignore:
+        if not self.unlocked and not self.previouslyUnlocked:
             self.unlocked = True
             self._sendUnlocked(self.player)
 
     def listeningForMessages(self):
-        return not self.unlocked and not self.ignore
+        return not self.unlocked and not self.previouslyUnlocked
 
-    def JSONString(self):
-        return {'unlocked' : self.unlocked or self.ignore,
-                'progress' : self.progress}
+    def getJsonString(self):
+        return {
+            'unlocked' : self.unlocked or self.previouslyUnlocked,
+            'progress' : self.progress,
+        }
 
 
 ###################
@@ -316,6 +325,8 @@ class EarnStarTrigger(PlayerKillTrigger, ZoneTagTrigger):
 ##########################
 
 class KilledSomeoneAchievement(Achievement):
+    messages = (PlayerKilledMsg,)
+
     def gotPlayerKilledMsg(self, msg):
         if msg.killerId == self.player.id:
             self.killedSomeone(msg)
@@ -331,7 +342,6 @@ class ChecklistAchievement(Achievement):
 
     def addItem(self, item):
         if item not in self.requiredItems:
-            #print 'Attempt to add weird item %r to %r' % (item, self)
             return
         self.progress.add(item)
 
@@ -352,23 +362,21 @@ class ChecklistAchievement(Achievement):
 
 class PersistedChecklistAchievement(ChecklistAchievement, OnceEverPerPlayer):
     def readExistingData(self):
-        if self.player.user is not None:
-            if self.player.user.achievements.has_key(self.idstring):
-                self.ignore = self.player.user.achievements[self.idstring].get(
-                        'unlocked', False)
-                self.progress = set(self.player.user.achievements[
-                        self.idstring].get('progress', []))
-            else:
-                self.ignore = False
-                self.progress = set([])
+        user = self.player.user
+        if user is None:
+            self.previouslyUnlocked = False
+            self.progress = set()
         else:
-            self.ignore = True
-
-    def JSONString(self):
-        return {'unlocked' : self.unlocked,
-                # JSON doesn't support sets:
-                'progress' : list(self.progress)}
+            userAchievements = user.achievements.get(self.idstring, {})
+            self.previouslyUnlocked = userAchievements.get('unlocked', False)
+            self.progress = set(userAchievements.get('progress', []))
 
+    def getJsonString(self):
+        return {
+            'unlocked' : self.unlocked,
+            # JSON doesn't support sets:
+            'progress' : list(self.progress),
+        }
 
 class TimedAchievement(Achievement):
     requiredValue = -1
@@ -411,6 +419,7 @@ class MultiTagSmall(Streak, OncePerPlayerPerGame):
     incrementTrigger = ZoneTagTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 3
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class MultiTagMedium(Streak, OncePerPlayerPerGame):
     idstring = 'multiTagMedium'
@@ -420,6 +429,7 @@ class MultiTagMedium(Streak, OncePerPlayerPerGame):
     incrementTrigger = ZoneTagTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 5
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class MultiTagLarge(Streak, OncePerPlayerPerGame):
     idstring = 'multiTagLarge'
@@ -429,6 +439,7 @@ class MultiTagLarge(Streak, OncePerPlayerPerGame):
     incrementTrigger = ZoneTagTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 8
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class TotalTagsSmall(Streak, OnceEverPerPlayer):
     idstring = 'totalTagsSmall'
@@ -437,6 +448,7 @@ class TotalTagsSmall(Streak, OnceEverPerPlayer):
 
     incrementTrigger = ZoneTagTrigger
     streakTarget = 20
+    messages = incrementTrigger.messages
 
 class TotalTagsMedium(Streak, OnceEverPerPlayer):
     idstring = 'totalTagsMedium'
@@ -445,6 +457,7 @@ class TotalTagsMedium(Streak, OnceEverPerPlayer):
 
     incrementTrigger = ZoneTagTrigger
     streakTarget = 50
+    messages = incrementTrigger.messages
 
 class MultiKillSmall(Streak, OncePerPlayerPerGame):
     idstring = 'multikillSmall'
@@ -454,6 +467,7 @@ class MultiKillSmall(Streak, OncePerPlayerPerGame):
     incrementTrigger = PlayerKillTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 3
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class MultiKillMedium(Streak, OncePerPlayerPerGame):
     idstring = 'multikillMedium'
@@ -463,6 +477,7 @@ class MultiKillMedium(Streak, OncePerPlayerPerGame):
     incrementTrigger = PlayerKillTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 5
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class MultiKillLarge(Streak, OncePerPlayerPerGame):
     idstring = 'multikillLarge'
@@ -472,6 +487,7 @@ class MultiKillLarge(Streak, OncePerPlayerPerGame):
     incrementTrigger = PlayerKillTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 10
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class TotalKillsSmall(Streak, OnceEverPerPlayer):
     idstring = 'totalKillsSmall'
@@ -480,6 +496,7 @@ class TotalKillsSmall(Streak, OnceEverPerPlayer):
 
     incrementTrigger = PlayerKillTrigger
     streakTarget = 50
+    messages = incrementTrigger.messages
 
 class TotalKillsMedium(Streak, OnceEverPerPlayer):
     idstring = 'totalKillsMedium'
@@ -488,6 +505,7 @@ class TotalKillsMedium(Streak, OnceEverPerPlayer):
 
     incrementTrigger = PlayerKillTrigger
     streakTarget = 100
+    messages = incrementTrigger.messages
 
 class ShoppingSpree(Streak, OncePerPlayerPerGame):
     idstring = 'multiUpgradesSmall'
@@ -497,6 +515,7 @@ class ShoppingSpree(Streak, OncePerPlayerPerGame):
     incrementTrigger = UseUpgradeTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 2
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class SmartShopper(Streak, OncePerPlayerPerGame):
     idstring = 'multiUpgradesMedium'
@@ -505,6 +524,7 @@ class SmartShopper(Streak, OncePerPlayerPerGame):
 
     incrementTrigger = UseUpgradeTrigger
     streakTarget = 5
+    messages = incrementTrigger.messages
 
 class BulletsSmall(Streak, OncePerPlayerPerGame):
     idstring = 'bulletsSmall'
@@ -514,6 +534,7 @@ class BulletsSmall(Streak, OncePerPlayerPerGame):
     incrementTrigger = FireShotTrigger
     resetTrigger = PlayerDeathTrigger
     streakTarget = 100
+    messages = incrementTrigger.messages + resetTrigger.messages
 
 class BulletsMedium(Streak, OncePerPlayerPerGame):
     idstring = 'bulletsMedium'
@@ -522,6 +543,7 @@ class BulletsMedium(Streak, OncePerPlayerPerGame):
 
     incrementTrigger = FireShotTrigger
     streakTarget = 500
+    messages = incrementTrigger.messages
 
 class AssistTags(Streak, OncePerPlayerPerGame):
     idstring = 'assistTags'
@@ -530,6 +552,7 @@ class AssistTags(Streak, OncePerPlayerPerGame):
 
     incrementTrigger = TagAssistTrigger
     streakTarget = 5
+    messages = incrementTrigger.messages
 
 class QuickKillSmall(QuickKillAchievement, NoLimit):
     requiredValue = 2
@@ -574,12 +597,14 @@ class EarnStars(IncrementingAchievement, OnceEverPerPlayer):
 
     incrementTrigger = EarnStarTrigger
     requiredQuantity = 100
+    messages = incrementTrigger.messages
 
 class BuyEveryUpgrade(PersistedChecklistAchievement):
     idstring = 'buyEveryUpgrade'
     name = 'Technology Whiz'
     description = 'Use every upgrade at least once'
-    requiredItems = set(['s', 'h', 't', 'm', 'g', 'r'])
+    requiredItems = set(u.upgradeType for u in allUpgrades)
+    messages = (PlayerHasUpgradeMsg,)
 
     @checkPlayerId
     def gotPlayerHasUpgradeMsg(self, msg):
@@ -589,6 +614,7 @@ class UseMinimapDisruption(OncePerPlayerPerGame):
     idstring = 'minimapDisruption'
     name = 'The Rader Appears to be... Jammed!'
     description = 'Use a minimap disruption'
+    messages = (PlayerHasUpgradeMsg,)
 
     @checkPlayerId
     def gotPlayerHasUpgradeMsg(self, msg):
@@ -599,13 +625,15 @@ class EarlyAbandon(OncePerPlayerPerGame):
     idstring = 'abandonEarly'
     name = 'Reduce, Reuse, Recycle'
     description = 'Abandon a turret or phase shift before it runs out'
-    # Don't award if it was within 3 seconds of running out anyway
+    messages = (DeleteUpgradeMsg,)
 
     @checkPlayerId
     def gotDeleteUpgradeMsg(self, msg):
-        if msg.deleteReason == 'A' \
-           and msg.upgradeType in 'th' \
-           and self.player.upgradeGauge >= 3:
+        '''
+        Don't award if it was within 3 seconds of running out anyway.
+        '''
+        if (msg.deleteReason == 'A' and msg.upgradeType in 'th' and
+                self.player.upgradeGauge >= 3):
             self.achievementUnlocked()
 
 class GoodManners(OncePerPlayerPerGame):
@@ -613,6 +641,7 @@ class GoodManners(OncePerPlayerPerGame):
     name = 'Good Manners'
     description = 'Say "gg" after the game finishes'
     afterGame = True
+    messages = (ChatMsg,)
 
     @checkPlayerId
     def gotChatMsg(self, msg):
@@ -638,6 +667,7 @@ class PhaseShiftAbsorb(IncrementingAchievement, OncePerPlayerPerGame):
     name = "Can't Touch This"
     description = 'Get hit by 30 bullets in a single phase shift'
     requiredQuantity = 30
+    messages = (ShotAbsorbedMsg, DeleteUpgradeMsg)
 
     def gotShotAbsorbedMsg(self, msg):
         if (msg.targetId == self.player.id and self.player.upgrade is not None
@@ -694,7 +724,8 @@ class RabbitKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
 
     def killedSomeone(self, msg):
         if (self.world.winningTeam == self.player.team and
-                self.player.isEnemyTeam(self.world.getPlayer(msg.targetId).team)):
+                self.player.isEnemyTeam(self.world.getPlayer(msg.targetId
+                ).team)):
             self.achievementUnlocked()
 
 class TurretKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
@@ -703,14 +734,15 @@ class TurretKill(KilledSomeoneAchievement, OncePerPlayerPerGame):
     idstring = 'turretKill'
 
     def killedSomeone(self, msg):
-        if self.player.upgrade is not None \
-           and self.player.upgrade.upgradeType == 't':
+        if (self.player.upgrade is not None and self.player.upgrade.upgradeType
+                == 't'):
             self.achievementUnlocked()
 
 class FirstKill(OncePerGame):
     name = 'First Blood'
     description = 'Make the first kill of the game'
     idstring = 'firstKill'
+    messages = (PlayerKilledMsg,)
 
     def gotPlayerKilledMsg(self, msg):
         if not self.unlocked:
@@ -726,6 +758,7 @@ class FirstTag(OncePerGame):
     name = 'Game On'
     description = 'Tag or assist in the tagging of the first zone in a game'
     idstring = 'firstTag'
+    messages = (TaggingZoneMsg,)
 
     def gotTaggingZoneMsg(self, msg):
         if not self.unlocked:
@@ -741,6 +774,7 @@ class LastTag(OncePerGame):
     name = 'And the Dirt is Gone'
     description = 'Tag or assist in the tagging of the final zone'
     idstring = 'finalTag'
+    messages = (TaggingZoneMsg,)
 
     def gotTaggingZoneMsg(self, msg):
         team = self.world.getTeam(msg.teamId)
@@ -773,15 +807,13 @@ class MinimapDisruptionTag(OncePerPlayerPerGame):
     name = 'Tag in the Dark'
     description = 'Tag a zone while the enemy\'s minimap is disrupted'
     idstring = 'disruptionTag'
-
-    # TODO: Do this by remembering if someone is using a minimap disruption
+    messages = (TaggingZoneMsg,)
 
     @checkPlayerId
     def gotTaggingZoneMsg(self, msg):
         for player in self.world.players:
-            if player.team == self.player.team \
-               and player.upgrade is not None \
-               and player.upgrade.upgradeType == 'm':
+            if (player.team == self.player.team and player.upgrade is not None
+                    and player.upgrade.upgradeType == 'm'):
                 self.achievementUnlocked()
                 break
 
@@ -789,6 +821,7 @@ class MutualKill(NoLimit):
     name = 'Eye for an Eye'
     description = 'Kill an enemy at the same time he kills you'
     idstring = 'mutualKill'
+    messages = (PlayerKilledMsg, RespawnMsg, RemovePlayerMsg)
 
     def __init__(self, player, world, outPlug):
         super(MutualKill, self).__init__(player, world, outPlug)
@@ -829,6 +862,7 @@ class SnatchedZone(OncePerPlayerPerGame):
     # anyway
     description = 'Tag a zone just before an enemy was about to respawn in it'
     snatchTime = 0.5
+    messages = (TaggingZoneMsg,)
 
     @checkPlayerId
     def gotTaggingZoneMsg(self, msg):
@@ -857,6 +891,7 @@ class KilledByNobody(KilledSomeoneAchievement, OncePerPlayerPerGame):
     name = 'And Who Deserves the Blame?'
     description = 'Die with no-one being awarded a kill'
     idstring = 'killedByNobody'
+    messages = (PlayerKilledMsg,)
 
     def gotPlayerKilledMsg(self, msg):
         if msg.targetId == self.player.id and msg.killerId == '\x00':
@@ -867,7 +902,8 @@ class ShieldRevenge(OncePerPlayerPerGame):
     description = 'Kill the enemy who shot your shield within ten seconds'
     idstring = 'shieldRevenge'
 
-    timeWindow = 10 #Seconds
+    timeWindow = 10 # Seconds
+    messages = (ShotAbsorbedMsg, PlayerKilledMsg)
 
     def __init__(self, player, world, outPlug):
         super(ShieldRevenge, self).__init__(player, world, outPlug)
@@ -906,7 +942,8 @@ class ShieldThenKill(OncePerPlayerPerGame):
     description = 'Kill an enemy after destroying their shields'
     idstring = 'destroyShield'
 
-    timeWindow = 10 #Seconds
+    timeWindow = 10 # Seconds
+    messages = (ShotAbsorbedMsg, PlayerKilledMsg)
 
     def __init__(self, player, world, outPlug):
         super(ShieldThenKill, self).__init__(player, world, outPlug)
@@ -945,6 +982,7 @@ class LastManStanding(OncePerTeamPerGame):
     description = 'Be the last surviving member of your team'
     idstring = 'surviveLast'
     afterGame = True
+    messages = (PlayerKilledMsg, GameOverMsg)
 
     def __init__(self, world, team, outPlug):
         super(LastManStanding, self).__init__(world, team, outPlug)
@@ -1017,6 +1055,8 @@ class KillEveryLeader(KilledSomeoneAchievement, PersistedChecklistAchievement):
 
     def killedSomeone(self, msg):
         player = self.world.getPlayer(msg.targetId)
+        if player.user is None:
+            return
         nick = stripPunctuation(player.user.username).lower()
         self.addItem(nick)
 
