@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2010 Joshua D Bartlett
+# Copyright (C) 2006-2011 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -115,8 +115,8 @@ class MapBlockDef(object):
             deltaY = y - self.pos[1] - MapLayout.halfZoneHeight
             deltaX = x - self.pos[0]
 
-            if deltaY * MapLayout.zoneInterfaceWidth > \
-                   -MapLayout.halfZoneHeight * deltaX:
+            if (deltaY * MapLayout.zoneInterfaceWidth >
+                   -MapLayout.halfZoneHeight * deltaX):
                 return self.zone2
             return self.zone1
         elif self.kind == 'bck':
@@ -125,8 +125,8 @@ class MapBlockDef(object):
             deltaY = y - self.pos[1]
             deltaX = x - self.pos[0]
 
-            if deltaY * MapLayout.zoneInterfaceWidth > \
-                   MapLayout.halfZoneHeight * deltaX:
+            if (deltaY * MapLayout.zoneInterfaceWidth >
+                    MapLayout.halfZoneHeight * deltaX):
                 return self.zone1
             return self.zone2
         else:
@@ -379,7 +379,7 @@ class MapBlock(object):
     def addCollectableStar(self, star):
         '''Adds a given grenade to this mapBlock'''
         self.collectableStars.add(star)
-        
+
 class BodyMapBlock(MapBlock):
     '''Represents a map block which contains only a single zone.'''
 
@@ -407,9 +407,9 @@ class TopBodyMapBlock(BodyMapBlock):
     def fromEdge(self, player):
         '''Returns the distance from the edge of the zone that a player is'''
 
-        return min(abs(player.pos[1] - self.defn.rect.top), \
-                   self.blockLeft.fromEdge(player),\
-                   self.blockRight.fromEdge(player))
+        return min(abs(player.pos[1] - self.defn.rect.top),
+                self.blockLeft.fromEdge(player),
+                self.blockRight.fromEdge(player))
 
 class BottomBodyMapBlock(BodyMapBlock):
     '''Represents a map block containing the bottom half of a zone.'''
@@ -420,9 +420,9 @@ class BottomBodyMapBlock(BodyMapBlock):
     def fromEdge(self, player):
         '''Returns the distance from the edge of the zone that a player is'''
 
-        return min(abs(self.defn.rect.bottom - player.pos[1]), \
-                   self.blockLeft.fromEdge(player),\
-                   self.blockRight.fromEdge(player))
+        return min(abs(self.defn.rect.bottom - player.pos[1]),
+                self.blockLeft.fromEdge(player),
+                self.blockRight.fromEdge(player))
 
 class InterfaceMapBlock(MapBlock):
     '''Base class for map blocks which contain the interface between two
@@ -456,20 +456,21 @@ class ForwardInterfaceMapBlock(InterfaceMapBlock):
         deltaY = y - self.defn.pos[1] - MapLayout.halfZoneHeight
         deltaX = x - self.defn.pos[0]
 
-        if deltaY * MapLayout.zoneInterfaceWidth > \
-               -MapLayout.halfZoneHeight * deltaX:
+        if (deltaY * MapLayout.zoneInterfaceWidth > -MapLayout.halfZoneHeight *
+                deltaX):
             return self.zone2
         return self.zone1
 
     def fromEdge(self, player):
         '''Returns the distance from the edge of the zone that a player is.
         Assumes that the player is actually in this map block.'''
-        relPos = (self.defn.rect.left - player.pos[0], self.defn.rect.top - player.pos[1])
+        relPos = (self.defn.rect.left - player.pos[0], self.defn.rect.top -
+                player.pos[1])
         # Note: this following formula relies upon the dimensions:
         # MapLayout.halfZoneHeight = 384
         # MapLayout.zoneInterfaceWidth = 512
         # Should they be changed, should change this to:
-        # d = sin(theta) * abs(relPos[1] + relPos[0] * \
+        # d = sin(theta) * abs(relPos[1] + relPos[0] *
         # MapLayout.halfZoneHeight / MapLayout.zoneInterfaceWidth + 384)
         # (where theta is the angle formed by the diagonal line seperating the
         # zones, and a vertical line).
@@ -492,21 +493,22 @@ class BackwardInterfaceMapBlock(InterfaceMapBlock):
         deltaY = y - self.defn.pos[1]
         deltaX = x - self.defn.pos[0]
 
-        if deltaY * MapLayout.zoneInterfaceWidth > \
-               MapLayout.halfZoneHeight * deltaX:
+        if (deltaY * MapLayout.zoneInterfaceWidth >
+               MapLayout.halfZoneHeight * deltaX):
             return self.zone1
         return self.zone2
 
     def fromEdge(self, player):
         '''Returns the distance from the edge of the zone that a player is.
         Assumes that the player is actually in this map block.'''
-        relPos = (self.defn.rect.left - player.pos[0], self.defn.rect.top - player.pos[1])
+        relPos = (self.defn.rect.left - player.pos[0], self.defn.rect.top -
+                player.pos[1])
 
         # Note: this following formula relies upon the dimensions:
         # MapLayout.halfZoneHeight = 384
         # MapLayout.zoneInterfaceWidth = 512
         # Should they be changed, should change this to:
-        # d = sin(theta) * abs(relPos[1] - relPos[0] * \
+        # d = sin(theta) * abs(relPos[1] - relPos[0] *
         # MapLayout.halfZoneHeight / MapLayout.zoneInterfaceWidth)
         # where theta is the angle formed by the diagonal line seperating the
         # zones, and a vertical line.
