@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2010 Joshua D Bartlett
+# Copyright (C) 2006-2011 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,14 +18,15 @@
 '''universe.py - defines anything that has to do with the running of the
 universe. This includes players, shots, zones, and the level itself.'''
 
+import logging
 from math import sin, cos
 import random
 
 from trosnoth.utils.utils import timeNow
-import trosnoth.utils.logging as logging
+from trosnoth.utils.math import distance
 from trosnoth.utils.checkpoint import checkpoint
 from trosnoth.model.universe_base import GameState, GameRules
-from trosnoth.model.upgrades import upgradeOfType
+from trosnoth.model.upgrades import upgradeOfType, allUpgrades
 from trosnoth.model.physics import WorldPhysics
 
 from trosnoth.model.map import MapLayout, MapState
@@ -33,12 +34,11 @@ from trosnoth.model.shot import Shot
 from trosnoth.model.player import Player
 from trosnoth.model.star import CollectableStar
 from trosnoth.model.team import Team
-from trosnoth.model.zone import ZoneState
-
 
 # Component message passing
 from trosnoth.utils.components import Component, handler, Plug
-from trosnoth.network.utils import expand_boolean
+from trosnoth.utils.network import expand_boolean
+from trosnoth.utils.twist import WeakCallLater, WeakLoopingCall
 from trosnoth.messages import (ChatMsg, TaggingZoneMsg, ShotFiredMsg,
         PlayerUpdateMsg, KillShotMsg, CannotRespawnMsg, RespawnMsg,
         PlayerKilledMsg, PlayerHitMsg, GameStartMsg, GameOverMsg, SetCaptainMsg,
@@ -50,10 +50,8 @@ from trosnoth.messages import (ChatMsg, TaggingZoneMsg, ShotFiredMsg,
         AchievementUnlockedMsg, ZoneStateMsg, WorldResetMsg,
         QueryWorldParametersMsg, PlayerIsReadyMsg, SetPreferredTeamMsg,
         SetPreferredDurationMsg, SetPreferredSizeMsg, CreateCollectableStarMsg,
-        RemoveCollectableStarMsg, PlayerHasElephantMsg, ChangeNicknameMsg)
-
-
-from twisted.internet import reactor, task
+        RemoveCollectableStarMsg, PlayerHasElephantMsg, ChangeNicknameMsg,
+        FireShoxwaveMsg, MarkZoneMsg, SetGameSpeedMsg)
 
 DIRTY_DELAY = 0.05
 UPDATE_DELAY = 1        # PlayerUpdateMsg is sent for each player this often.
@@ -71,6 +69,8 @@ NO_COLLECTABLE_STARS = False        # For debugging.
 # collectable star appearing every STALEMATE_CHECK_PERIOD.
 ZONE_TAG_STAR_HALF_LIFE = 60
 
+log = logging.getLogger('universe')
+
 class Abort(Exception):
     pass
 
@@ -80,10 +80,6 @@ class Universe(Component):
     and states of every alien, the terrain positions, and who owns the
     various territories and orbs.'''
 
-    # The size of players and shots.
-    halfPlayerSize = (10, 19)
-    halfShotSize = (5, 5)
-
     # eventPlug is a sending plug
     # Events which are generated by the tick() method of the universe
     # should be sent to this plug. This indicates that the universe thinks
@@ -99,7 +95,7 @@ class Universe(Component):
 
     def __init__(self, halfMapWidth=3, mapHeight=2, gameDuration=0,
             authTagManager=None, voting=False, layoutDatabase=None,
-            clientOptimised=False, gameName=None):
+            clientOptimised=False, gameName=None, onceOnly=False):
         '''
         halfMapWidth:   is the number of columns of zones in each team's
                         territory at the start of the game. There will always
@@ -124,6 +120,7 @@ class Universe(Component):
         self.physics = WorldPhysics(self)
         self.gameDuration = gameDuration
         self.layoutDatabase = layoutDatabase
+        self.onceOnly = onceOnly
         if authTagManager is None:
             self.authManager = None
         else:
@@ -163,8 +160,6 @@ class Universe(Component):
         self.playerUpdateTimes = {}
         self.grenades = set()
         self.collectableStars = {}      # starId -> CollectableStar
-        # TODO: could make shots iterate directly over the player objects rather than storing twice
-        # (redundancy - potential inconsistencies)
         self.shots = set()
         self.gameOverTime = None
         self.gameMode = 'Normal'
@@ -189,6 +184,16 @@ class Universe(Component):
     def ignore(self, msg):
         pass
 
+    @handler(MarkZoneMsg, orderPlug)
+    def markZone(self, msg):
+        try:
+            zone = self.getZone(msg.zoneId)
+            player = self.getPlayer(msg.playerId)
+
+            zone.markedBy[player.team] = msg.value
+        except Abort:
+            pass
+
     @handler(AchievementUnlockedMsg, orderPlug)
     def achievementUnlocked(self, msg):
         try:
@@ -197,7 +202,7 @@ class Universe(Component):
                 player.user.achievementUnlocked(msg)
         except Abort:
             pass
-            
+
     def reset(self, layout, gameDuration=None):
         if gameDuration is not None:
             self.gameDuration = gameDuration
@@ -220,8 +225,8 @@ class Universe(Component):
         i, j = MapLayout.getMapBlockIndices(*player.pos)
         try:
             block = self.zoneBlocks[i][j]
-        except IndexError:
-            logging.writeException()
+        except IndexError, e:
+            log.exception(str(e))
             raise IndexError, 'player start position is off the map'
         player.setMapBlock(block)
 
@@ -231,9 +236,8 @@ class Universe(Component):
         actionZones = []
         for zone in list(allTeamZones):
             for adj in zone.defn.adjacentZones.iterkeys():
-                if adj is not None and \
-                   (self.zoneWithDef[adj].orbOwner is None or \
-                   self.zoneWithDef[adj].orbOwner.id != teamId):
+                if (adj is not None and (self.zoneWithDef[adj].orbOwner is None
+                        or self.zoneWithDef[adj].orbOwner.id != teamId)):
                     actionZones.append(zone)
         if len(actionZones) > 0:
             return random.choice(actionZones)
@@ -341,7 +345,15 @@ class Universe(Component):
         mode = msg.gameMode.decode()
         if self.physics.setMode(mode):
             self.gameMode = mode
-            print 'Client: GameMode is set to ' + mode
+            log.debug('Client: GameMode is set to ' + mode)
+
+    @handler(SetGameSpeedMsg, orderPlug)
+    def setGameSpeed(self, msg):
+        '''Sets the speed of the game to a proportion of normal speed.
+        That is, speed=2.0 is twice as fast a game as normal
+        '''
+        speed = msg.gameSpeed
+        self._gameSpeed = speed
 
     @handler(SetTeamNameMsg, orderPlug)
     def setTeamName(self, msg):
@@ -371,7 +383,7 @@ class Universe(Component):
             player.readyToStart = False
             if player.ghost and player.team is not None:
                 # Give a chance for stats to save
-                reactor.callLater(0.5, self.eventPlug.send,
+                WeakCallLater(0.5, self.eventPlug, 'send',
                         SetPlayerTeamMsg(player.id, '\x00'))
 
     def setHuntedMode(self, winningTeam):
@@ -380,9 +392,9 @@ class Universe(Component):
         else:
             huntTeamWithZones = winningTeam
         self.setHuntTeamWithZones(huntTeamWithZones)
-        
+
     def setHuntTeamWithZones(self, huntTeamWithZones):
-        print 'Trying to hunt with %s' % (huntTeamWithZones,)
+        log.debug('Trying to hunt with %s' % (huntTeamWithZones,))
         # Firstly, if somehow all players are on the one zone, swap one to the
         # other team
         if len(self.players) < 4:
@@ -397,9 +409,9 @@ class Universe(Component):
             player = random.choice(self.players)
             team = list(allTeams)[0].opposingTeam
             self.eventPlug.send(SetPlayerTeamMsg(player.id, team.id))
-            reactor.callLater(3, self.setHuntedMode, team)
+            WeakCallLater(3, self, 'setHuntedMode', team)
         else:
-            # Now, check by who is alive and dead on each team        
+            # Now, check by who is alive and dead on each team
             playersWithoutZones = 0
             playersWithZones = 0
             playersToSpawn = []
@@ -415,22 +427,25 @@ class Universe(Component):
                     else:
                         playersWithZones += 1
             if playersWithoutZones < 2: # Having only one player won't work
-                reactor.callLater(3, self.setHuntedMode, huntTeamWithZones.opposingTeam)
+                WeakCallLater(3, self, 'setHuntedMode',
+                        huntTeamWithZones.opposingTeam)
             else:
-                print 'Hunting!'
+                log.debug('Hunting!')
                 self.huntTeamWithZones = huntTeamWithZones
                 self.gameRules = GameRules.Hunting
                 if playersWithZones == 0:
                     # We're about to swap everyone onto the one team. Stop this!
                     playersToSwapTeam.remove(random.choice(playersToSwapTeam))
-                    
+
                 for p in playersToSwapTeam:
-                    self.eventPlug.send(SetPlayerTeamMsg(p.id, huntTeamWithZones.opposingTeam.id))
+                    self.eventPlug.send(SetPlayerTeamMsg(p.id,
+                            huntTeamWithZones.opposingTeam.id))
                 for p in playersToSpawn:
                     self.eventPlug.send(RespawnMsg(p.id, player.currentZone.id))
                 for zone in self.zones:
-                    self.eventPlug.send(ZoneStateMsg(zone.id, self.huntTeamWithZones.id, True))
-            
+                    self.eventPlug.send(ZoneStateMsg(zone.id,
+                            self.huntTeamWithZones.id, True, ''))
+
 
 
     @handler(GameStartMsg, orderPlug)
@@ -459,7 +474,7 @@ class Universe(Component):
             player.team = team
         except Abort:
             pass
-        
+
 
     def _makePlayer(self, nick, team, playerId, zone, dead, bot):
         # Check if this is a duplicate msg.
@@ -477,8 +492,8 @@ class Universe(Component):
         i, j = MapLayout.getMapBlockIndices(*player.pos)
         try:
             block = self.zoneBlocks[i][j]
-        except IndexError:
-            logging.writeException()
+        except IndexError, e:
+            log.exception(str(e))
             raise IndexError, 'player start position is off the map'
         player.setMapBlock(block)
 
@@ -510,7 +525,7 @@ class Universe(Component):
     def _startClock(self):
         if self._loop is not None:
             self._loop.stop()
-        self._loop = task.LoopingCall(self.tick)
+        self._loop = WeakLoopingCall(self, 'tick')
         self._loop.start(TICK_PERIOD, False)
 
     def stopClock(self):
@@ -529,7 +544,7 @@ class Universe(Component):
         self._lastTime = t
         gameDeltaT = (deltaT * self._gameSpeed)
         self._gameTime += gameDeltaT
-        
+
         # Update the player and shot positions.
         for s in (list(self.shots) + list(self.players) + list(self.grenades) +
                 self.collectableStars.values()):
@@ -547,12 +562,6 @@ class Universe(Component):
             self.checkStalemate()
             self.updateCollectableStars()
 
-    def setGameSpeed(self, speed):
-        '''Sets the speed of the game to a proportion of normal speed.
-        That is, speed=2.0 is twice as fast a game as normal
-        '''
-        self._gameSpeed = speed
-
     def getGameTime(self):
         '''The amount of time since the start of the game.
         This is influnced by any game speed factors
@@ -563,7 +572,7 @@ class Universe(Component):
         for star in self.collectableStars.values():
             if star.creationTime < self._gameTime - COLLECTABLE_STAR_LIFETIME:
                 self.eventPlug.send(RemoveCollectableStarMsg(star.id, '\x00'))
-        
+
     def checkStalemate(self):
         if NO_COLLECTABLE_STARS:
             return
@@ -576,9 +585,10 @@ class Universe(Component):
             ePlayerId = self.playerWithElephant.id
         if self.gameState != GameState.InProgress:
             return
-        
+
         self.eventPlug.send(PlayerHasElephantMsg(ePlayerId))
-        prob = 0.5 ** ((self._gameTime - self.lastZoneTagged) / ZONE_TAG_STAR_HALF_LIFE)
+        prob = 0.5 ** ((self._gameTime - self.lastZoneTagged) /
+                ZONE_TAG_STAR_HALF_LIFE)
         if random.random() > prob:
             team = random.choice(self.teams)
             zone = self.selectZone(team.id)
@@ -587,8 +597,8 @@ class Universe(Component):
 
     @handler(CreateCollectableStarMsg, orderPlug)
     def createCollectableStar(self, msg):
-        self.addCollectableStar(CollectableStar(self, msg.starId, (msg.xPos, msg.yPos),
-                self.getTeam(msg.teamId)))
+        self.addCollectableStar(CollectableStar(self, msg.starId, (msg.xPos,
+                msg.yPos), self.getTeam(msg.teamId)))
 
     @handler(RemoveCollectableStarMsg, orderPlug)
     def gotRemoveCollectableStarMsg(self, msg):
@@ -654,13 +664,11 @@ class Universe(Component):
             checkpoint('Universe: Zones captured, red wins')
 
     def processGameState(self):
-        if self.gameState != GameState.Ended:
+        if self.gameState != GameState.Ended or self.onceOnly:
             return
         if self._gameTime < self._lastGameStateCheck + GAME_STATE_CHECK_PERIOD:
             return
         self._lastGameStateCheck = self._gameTime
-        if len(self.players) <= 1:
-            return
         if self.layoutDatabase is None:
             return
 
@@ -671,6 +679,8 @@ class Universe(Component):
                 actualPlayerNum += 1
                 if player.readyToStart:
                     readyPlayerCount += 1
+        if actualPlayerNum <= 1:
+            return
         if not readyPlayerCount >= 0.8 * actualPlayerNum:
             return
 
@@ -695,10 +705,10 @@ class Universe(Component):
             team = self.teams[1]
             self.eventPlug.send(SetPlayerTeamMsg(player.id, team.id))
             player.team = team
-            
+
         layout = self.layoutDatabase.generateRandomMapLayout(size[0], size[1])
         self.reset(layout, duration)
-        reactor.callLater(0.1, self._startSoon, 12)
+        WeakCallLater(0.1, self, '_startSoon', 12)
 
     def _getNewDuration(self, mapSize):
         results = {}
@@ -819,10 +829,38 @@ class Universe(Component):
             return self.rogueTeamName
         return self.getTeam(id).teamName
 
+    @handler(FireShoxwaveMsg, orderPlug)
+    def shoxwaveExplosion(self, msg):
+    	radius = 128
+    	# Get the player who fired this shoxwave
+    	shoxPlayer = self.getPlayer(msg.playerId)
+        self._update_reload_time(shoxPlayer)
+
+        # Loop through all the players in the game
+        for player in self.players:
+            if (not player.isFriendsWith(shoxPlayer) and distance(player.pos,
+                    shoxPlayer.pos) <= radius and not player.ghost):
+                player.deathDetected(shoxPlayer.id, 0, 'W')
+
+        for shot in list(self.shots):
+            if (not shot.originatingPlayer.isFriendsWith(shoxPlayer) and
+                    distance(shot.pos, shoxPlayer.pos) <= radius):
+                self.shotExpired(shot)
+
+        for star in list(self.collectableStars.itervalues()):
+            if (not shoxPlayer.isFriendsWithTeam(star.team) and
+                    distance(star.pos, shoxPlayer.pos) <= radius):
+               	star.delete()
+
+
     @handler(ShotFiredMsg, orderPlug)
     def shotFired(self, msg):
         '''A player has fired a shot.'''
-        player = self.playerWithId[msg.playerId]
+        try:
+            player = self.playerWithId[msg.playerId]
+        except KeyError:
+            return
+
         team = player.team
         pos = (msg.xpos, msg.ypos)
         turret = msg.type in ('T', 'G')
@@ -844,18 +882,30 @@ class Universe(Component):
         shot.
         '''
         if player.turret:
-            player.reloadTime = self.physics.playerTurretReloadTime
+            reloadTime = self.physics.playerTurretReloadTime
             player.turretHeat += self.physics.playerShotHeat
             if player.turretHeat > self.physics.playerTurretHeatCapacity:
                 player.turretOverHeated = True
+        elif player.machineGunner:
+            player.mgBulletsRemaining -= 1
+            if player.mgBulletsRemaining > 0:
+                reloadTime = self.physics.playerMachineGunFireRate
+            elif player.mgBulletsRemaining == 0:
+                reloadTime = self.physics.playerMachineGunReloadTime
+            else:
+                player.mgBulletsRemaining = 15
+                reloadTime = 0
+        elif player.shoxwave:
+            reloadTime = self.physics.playerShoxwaveReloadTime
         elif player.team is None:
-            player.reloadTime = self.physics.playerNeutralReloadTime
+            reloadTime = self.physics.playerNeutralReloadTime
         elif player.currentZone.zoneOwner == player.team:
-            player.reloadTime = self.physics.playerOwnReloadTime
+            reloadTime = self.physics.playerOwnReloadTime
         elif player.currentZone.zoneOwner is None:
-            player.reloadTime = self.physics.playerNeutralReloadTime
+            reloadTime = self.physics.playerNeutralReloadTime
         else:
-            player.reloadTime = self.physics.playerEnemyReloadTime
+            reloadTime = self.physics.playerEnemyReloadTime
+        player.reloadTime = player.reloadFrom = reloadTime
 
     def _getNewShotId(self, player, _ids={}):
         '''
@@ -979,34 +1029,11 @@ class Universe(Component):
         xCoord1, yCoord1 = tagger.pos
         xCoord2, yCoord2 = zone.defn.pos
 
-        distance = ((xCoord1 - xCoord2) ** 2 + (yCoord1 - yCoord2) ** 2) ** 0.5
-        if distance < tagDistance:
-            # Check to ensure the team owns an adjacent orb
-            found = False
-            for adjZoneDef in zone.defn.adjacentZones:
-                adjZone = self.zoneWithDef[adjZoneDef]
-                if adjZone.orbOwner == tagger.team:
-                    found = True
-                    break
-            if not found:
-                return
-
-            # Check to see if the team has sufficient numbers to take the zone.
-            numTaggers = 0
-            numDefenders = 0
-            for player in zone.players:
-                if player.turret:
-                    # Turreted players do not count as a player for the purpose
-                    # of reckoning whether an enemy can capture the orb
-                    continue
-                if player.team == tagger.team:
-                    numTaggers += 1
-                else:
-                    numDefenders += 1
-
-            if ZoneState.canTag(numTaggers, numDefenders):
-
-                if (numDefenders > 3 and
+        dist = ((xCoord1 - xCoord2) ** 2 + (yCoord1 - yCoord2) ** 2) ** 0.5
+        if dist < tagDistance:
+            attackingTeams = zone.getAttackingTeams()
+            if tagger.team in attackingTeams:
+                if (len(attackingTeams) > 1 and
                         zone.checkForOpposingTag(tagger.team)):
                     # The other team has also tagged it
                     if zone.orbOwner != None:
@@ -1065,13 +1092,17 @@ class Universe(Component):
                     i = 0
                     # Check number of remaining players on that team
                     for player in self.players:
-                        if player.team != self.huntTeamWithZones and player is not target:
+                        if (player.team != self.huntTeamWithZones and player is
+                                not target):
                             i += 1
                     if i > 1:
-                        self.eventPlug.send(SetPlayerTeamMsg(target.id, self.huntTeamWithZones.id))
+                        self.eventPlug.send(SetPlayerTeamMsg(target.id,
+                                self.huntTeamWithZones.id))
                     else:
-                        # The second last player on that team just died; change all zones now
-                        self.setHuntTeamWithZones(self.huntTeamWithZones.opposingTeam)
+                        # The second last player on that team just died; change
+                        # all zones now
+                        self.setHuntTeamWithZones(
+                                self.huntTeamWithZones.opposingTeam)
             if shot is not None:
                 self.removeShot(shot)
         except Abort:
@@ -1091,7 +1122,7 @@ class Universe(Component):
     def zoneOwned(self, msg):
         zone = self.map.zoneWithId[msg.zoneId]
         team = self.teamWithId[msg.teamId]
-        zone.setOwnership(team, msg.dark)
+        zone.setOwnership(team, msg.dark, msg.marks)
 
     @handler(RespawnMsg, orderPlug)
     def respawn(self, msg):
@@ -1123,7 +1154,7 @@ class Universe(Component):
             player.upgradeTotal = upgradeKind.timeRemaining
             player.upgrade.use()
         except Abort:
-            print 'Upgrade unable to be given to player %s' % (msg.playerId,)
+            log.debug('Upgrade unable to be given to player %r', msg.playerId)
 
     @handler(DeleteUpgradeMsg, orderPlug)
     def delUpgrade(self, msg):
@@ -1141,10 +1172,10 @@ class Universe(Component):
 
     def addCollectableStar(self, star):
         self.collectableStars[star.id] = star
-        
+
     def removeCollectableStar(self, star):
         del self.collectableStars[star.id]
-        
+
     @handler(SetCaptainMsg, orderPlug)
     def setCaptain(self, msg):
         try:
@@ -1162,7 +1193,7 @@ class Universe(Component):
             if player.team is not None and not player.team.ready:
                 player.team.ready = True
                 if self._gameShouldStart():
-                    reactor.callLater(0.1, self._startSoon)
+                    WeakCallLater(0.1, self, '_startSoon')
         except Abort:
             pass
 
@@ -1178,13 +1209,18 @@ class Universe(Component):
 
     def _startSoon(self, delay=7):
         self.eventPlug.send(StartingSoonMsg(delay))
-        reactor.callLater(delay, self.eventPlug.send,
+        WeakCallLater(delay, self.eventPlug, 'send',
                 GameStartMsg(self.gameDuration))
 
     @handler(StartingSoonMsg, orderPlug)
     def startingSoon(self, msg):
         self.gameState = GameState.Starting
 
+        # Kick all AI players now, the game is about to begin.
+        for p in list(self.players):
+            if p.bot:
+                self.eventPlug.send(RemovePlayerMsg(p.id))
+
     @handler(KillShotMsg, orderPlug)
     @handler(ShotAbsorbedMsg, orderPlug)
     def shotDestroyed(self, msg):
@@ -1288,7 +1324,7 @@ class Universe(Component):
 
     def setTestMode(self):
         self.PLAYER_RESET_STARS = 20
-        for upgradetype in upgradeOfType.itervalues():
+        for upgradetype in allUpgrades:
             upgradetype.requiredStars = 1
 
     @handler(PlayerIsReadyMsg, orderPlug)
