@@ -1,3 +1,21 @@
+# Trosnoth (UberTweak Platform Game)
+# Copyright (C) 2006-2011 Joshua D Bartlett
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# version 2 as published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+# 02110-1301, USA.
+
+import logging
 import struct
 
 from twisted.internet import reactor
@@ -5,7 +23,10 @@ from twisted.internet.protocol import Factory, DatagramProtocol
 from twisted.internet.error import CannotListenError
 from twisted.protocols.basic import Int32StringReceiver
 
-from trosnoth.network import netmsg
+from trosnoth.utils import netmsg
+from trosnoth.utils.twist import WeakCallLater
+
+log = logging.getLogger('netman')
 
 # Special network messages used by the Network Manager have id strings which
 # begin with '\x07' in order to make it unlikely that their id strings will
@@ -16,7 +37,7 @@ class NotifyUDPInfo(netmsg.NetworkMessage):
     fields = 'udpPort', 'remoteId'
     packspec = 'II'
     idString = '\x07RId'
-    
+
 class UDPPing(netmsg.NetworkMessage):
     fields = 'payload'
     packspec = 'I'
@@ -44,8 +65,8 @@ class UDPProtocol(DatagramProtocol):
             self.port = reactor.listenUDP(port, self)
         except CannotListenError:
             self.port = reactor.listenUDP(0, self)
-        
-    # To join a multicast group: 
+
+    # To join a multicast group:
     # self.transport.joinGroup(multicastGroup)
 
     def send(self, string, address):
@@ -76,7 +97,7 @@ class TCPReceiver(Int32StringReceiver):
 
     def sendMessage(self, msg):
         self.sendString(msg.pack())
-        
+
     def connectionLost(self, reason):
         self.factory._lostConnection(self)
 
@@ -100,7 +121,7 @@ class TCPReceiver(Int32StringReceiver):
                     self.handler.receiveBadString(self.connectionId, line)
                 else:
                     remote = self.transport.getPeer()
-                    print '%s: bad string from %s: %r' % (
+                    log.warning('%s: bad string from %s: %r',
                             self.handler.__class__.__name__,
                             (remote.host, remote.port), line)
                 return
@@ -139,7 +160,7 @@ class TCPReceiver(Int32StringReceiver):
         self.udpValid = True
 
         # Ping again in 30 seconds, just to be sure.
-        self.udpPinger = reactor.callLater(30, self.checkUdp)
+        self.udpPinger = WeakCallLater(30, self, 'checkUdp')
         self.curPingValue += 1  # Just to be sure it doesn't match.
         self.pingDelay = 1      # For next time the ping fails.
 
@@ -158,7 +179,7 @@ class TCPReceiver(Int32StringReceiver):
 
         # Back off, but still ping again.
         self.pingDelay = min(30, self.pingDelay * 1.5)
-        self.udpPinger = reactor.callLater(self.pingDelay, self.checkUdp)
+        self.udpPinger = WeakCallLater(self.pingDelay, self, 'checkUdp')
         self.curPingValue += 1  # Just to be sure it doesn't match.
 
     def checkUdp(self):
@@ -173,13 +194,13 @@ class TCPReceiver(Int32StringReceiver):
         # Make sure that no-one else will call this function soon.
         if self.udpPinger is not None and self.udpPinger.active():
             self.udpPinger.cancel()
-            
+
         # Select a ping value and send ping.
         self.curPingValue = (self.curPingValue + 1) % (2 ** 32)
         self.factory._sendUdpPing(self)
 
         # Set a timeout of 1 second on the ping.
-        self.udpPinger = reactor.callLater(0.5, self.udpPingFailed)
+        self.udpPinger = WeakCallLater(0.5, self, 'udpPingFailed')
 
 class NetworkManager(Factory):
     '''
@@ -258,14 +279,15 @@ class NetworkManager(Factory):
 
     def addHandler(self, handler):
         if handler.greeting in self._handlers:
-            raise KeyError('handler already registered with greeting %r' % (handler.greeting,))
+            raise KeyError('handler already registered with greeting %r' %
+                    (handler.greeting,))
         self._handlers[handler.greeting] = handler
 
     def removeHandler(self, handler):
         if self._handlers.get(handler.greeting) != handler:
             raise ValueError('handler not registered yet')
         del self._handlers[handler.greeting]
-        
+
     ##############
     # Connections
     ##############
@@ -306,7 +328,7 @@ class NetworkManager(Factory):
             connection = self._connections[connectionId]
         except KeyError:
             raise ValueError('%d is not a valid connectionId' % (connectionId,))
-        
+
         if connection is None:
             raise ValueError('connection not completely initialised yet')
 
@@ -334,10 +356,10 @@ class NetworkManager(Factory):
             connection = self._connections[connectionId]
         except KeyError:
             raise ValueError('%r is not a valid connectionId' % (connectionId,))
-        
+
         if connection is None:
             raise ValueError('connection not completely initialised yet')
-        
+
         udpInfo = connection.udpInfo
         if udpInfo is None or not connection.udpValid:
             # Revert to TCP if UDP info is not yet received or UDP pings have
@@ -373,15 +395,15 @@ class NetworkManager(Factory):
         Note that if you want to send a message using UDP but fall back to TCP
         if it's not verified yet, you can use the .send() method.
         '''
-        
+
         try:
             connection = self._connections[connectionId]
         except KeyError:
             raise ValueError('%d is not a valid connectionId' % (connectionId,))
-        
+
         if connection is None:
             raise ValueError('connection not completely initialised yet')
-        
+
         udpInfo = connection.udpInfo
         if udpInfo is None or not connection.udpValid:
             # UDP info is not yet received or UDP pings have not yet got
@@ -412,7 +434,8 @@ class NetworkManager(Factory):
         except KeyError:
             # Should never happen, but it does.
             # There's not much we can do here since we can't get the connId.
-            print 'NetworkManager: clientConnectionFailed without _attempting'
+            log.warning('NetworkManager: clientConnectionFailed without '
+                    '_attempting')
             return
 
         del self._attempting[connector]
@@ -524,7 +547,7 @@ class NetworkManager(Factory):
     def _sendUdpPing(self, connection):
         '''
         Called by a connection when it wants to send a UDP ping to its other
-        end. This should only be called once the UDP info from the other end 
+        end. This should only be called once the UDP info from the other end
         has been received.
         The payload of the ping is taken from connection.curPingValue.
         '''
