@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2010 Joshua D Bartlett
+# Copyright (C) 2006-2011 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,11 +15,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
+import logging
+
 import pygame
-from trosnoth.gui.common import (Region, Canvas, Location)
+
+from trosnoth.gui.common import (Region, Canvas, Location, ScaledSize)
 from trosnoth.gui.framework.elements import TextElement
 from trosnoth.gui.framework import framework, prompt
-from trosnoth.gui.framework.dialogbox import DialogBox
+from trosnoth.gui.framework.dialogbox import DialogBox, OkBox
 from trosnoth.gui.framework.elements import (SolidRect, TextButton)
 from trosnoth.gui.framework.tabContainer import TabContainer
 from trosnoth.gui.framework.tab import Tab
@@ -28,19 +31,32 @@ from trosnoth.network.lobby import (Lobby, AuthenticationCancelled,
 from trosnoth.network import authcommands
 from trosnoth.network.client import ConnectionFailed
 from trosnoth.utils.event import Event
-from trosnoth.utils import logging
+from trosnoth.utils.twist import WeakCallLater
+import trosnoth.version
 from twisted.internet import defer
 from twisted.internet.error import ConnectError
 from twisted.protocols import amp
 
-KNOWN_SERVERS = [('localhost', 6787), ('play.trosnoth.org', 6787)]
+KNOWN_SERVERS = [
+    'self',     # This instance of Trosnoth.
+    ('localhost', 6787),
+]
+if trosnoth.version.release:
+    KNOWN_SERVERS.append(('play.trosnoth.org', 6787))
+
+KNOWN_SERVERS.extend([
+    'others',   # Other games known by servers.
+    'lan',      # LAN games.
+    'create',   # Create a game on servers.
+])
+
+log = logging.getLogger('playscreen')
 
 class PlayAuthScreen(framework.CompoundElement):
-    def __init__(self, app, onClose=None):
+    def __init__(self, app, onSucceed=None, onFail=None):
         super(PlayAuthScreen, self).__init__(app)
-        self.onClose = Event()
-        if onClose is not None:
-            self.onClose.addListener(onClose)
+        self.onSucceed = Event(onSucceed)
+        self.onFail = Event(onFail)
 
         if app.displaySettings.useAlpha:
             alpha = 192
@@ -58,6 +74,7 @@ class PlayAuthScreen(framework.CompoundElement):
         cancel = TextButton(app, Location(Canvas(512, 624), 'midbottom'),
                 'Cancel', font, app.theme.colours.secondMenuColour,
                 app.theme.colours.white, onClick=self.cancel)
+        self.cancelled = False
         self.elements = [bg, self.logBox, cancel]
         self.passwordGetter = None
 
@@ -67,69 +84,112 @@ class PlayAuthScreen(framework.CompoundElement):
         return self.passwordGetter
 
     @defer.deferredGenerator
-    def begin(self):
+    def begin(self, servers=None, canHost=True):
+        self.cancelled = False
+        self._makePasswordGetter().cancelled = False
         lobby = Lobby(self.app, self.app.netman)
-        servers = list(KNOWN_SERVERS)
+        if servers is None:
+            servers = list(KNOWN_SERVERS)
 
-        if self.app.server is not None:
-            self.onClose.execute()
-            self.app.interface.connectToLocalServer()
-            return
+        badServers = set()
 
-        for server in list(servers):
-            self.logBox.log('Requesting games from %s:%d...' % server)
-            try:
-               games = defer.waitForDeferred(lobby.getGames(server))
-               yield games
-               games = games.getResult()
-            except ConnectError:
-                self.logBox.log('Unable to connect.')
-                servers.remove(server)
-            except amp.UnknownRemoteError:
-                self.logBox.log('Error on remote server.')
-            except amp.RemoteAmpError:
-                self.logBox.log('Error on remote server.')
-                logging.writeException()
-            except IOError:
-                self.logBox.log('Error connecting to remote server.')
-                logging.writeException()
-                servers.remove(server)
-            else:
-                if len(games) > 0:
-                    for game in games:
-                        try:
-                            self.logBox.log('Found game: joining.')
-                            result = defer.waitForDeferred(game.join(
-                                    self._makePasswordGetter()))
-                            yield result
-                            result = result.getResult()
-                        except authcommands.GameDoesNotExist:
-                            pass
-                        except AuthenticationCancelled:
-                            pass
-                        except authcommands.NotAuthenticated:
-                            self.logBox.log('Authentication failure.')
-                        except amp.UnknownRemoteError:
-                            self.logBox.log('Error on remote server.')
-                        except amp.RemoteAmpError:
-                            self.logBox.log('Error on remote server.')
-                            logging.writeException()
-                        else:
-                            self._joined(result)
-                            return
+        for server in servers:
+            if self.cancelled:
+                break
+
+            if server == 'self':
+                if self.app.server is not None:
+                    self.onSucceed.execute()
+                    self.app.interface.connectToLocalServer()
+                    return
+            elif isinstance(server, tuple):
+                if server in badServers:
+                    continue
+                self.logBox.log('Requesting games from %s:%d...' % server)
+                try:
+                   games = defer.waitForDeferred(lobby.getGames(server))
+                   yield games
+                   games = games.getResult()
+                except ConnectError:
+                    self.logBox.log('Unable to connect.')
+                    badServers.add(server)
+                except amp.UnknownRemoteError:
+                    self.logBox.log('Error on remote server.')
+                except amp.RemoteAmpError, e:
+                    self.logBox.log('Error on remote server.')
+                    log.exception(str(e))
+                except IOError, e:
+                    self.logBox.log('Error connecting to remote server.')
+                    log.exception(str(e))
+                    badServers.add(server)
                 else:
-                    self.logBox.log('No running games.')
-
-        for server in list(servers):
-            self.logBox.log('Asking %s:%d about other games...' % server)
-            try:
-               games = defer.waitForDeferred(lobby.getOtherGames(server))
-               yield games
-               games = games.getResult()
-            except ConnectError:
-                self.logBox.log('Unable to connect.')
-                servers.remove(server)
-            else:
+                    if len(games) > 0:
+                        for game in games:
+                            try:
+                                self.logBox.log('Found game: joining.')
+                                result = defer.waitForDeferred(game.join(
+                                        self._makePasswordGetter()))
+                                yield result
+                                result = result.getResult()
+                            except authcommands.GameDoesNotExist:
+                                pass
+                            except AuthenticationCancelled:
+                                pass
+                            except authcommands.NotAuthenticated:
+                                self.logBox.log('Authentication failure.')
+                            except amp.UnknownRemoteError:
+                                self.logBox.log('Error on remote server.')
+                            except amp.RemoteAmpError, e:
+                                self.logBox.log('Error on remote server.')
+                                log.exception(str(e))
+                            else:
+                                self._joined(result)
+                                return
+                    else:
+                        self.logBox.log('No running games.')
+            elif server == 'others':
+                for server in servers:
+                    if server in badServers or not isinstance(server, tuple):
+                        continue
+                    self.logBox.log('Asking %s:%d about other games...' %
+                            server)
+                    try:
+                        games = defer.waitForDeferred(lobby.getOtherGames(
+                            server))
+                        yield games
+                        games = games.getResult()
+                    except ConnectError:
+                        self.logBox.log('Unable to connect.')
+                        badServers.add(server)
+                    else:
+                        for game in games:
+                            try:
+                                self.logBox.log('Found game: joining.')
+                                result = defer.waitForDeferred(game.join(
+                                        self._makePasswordGetter()))
+                                yield result
+                                result = result.getResult()
+                            except authcommands.GameDoesNotExist:
+                                pass
+                            except amp.UnknownRemoteError:
+                                self.logBox.log('Error on remote server.')
+                            except amp.RemoteAmpError, e:
+                                self.logBox.log('Error on remote server.')
+                                log.exception(str(e))
+                            except IOError, e:
+                                self.logBox.log(
+                                        'Error connecting to remote server.')
+                                log.exception(str(e))
+                            except ConnectionFailed:
+                                self.logBox.log('Could not connect.')
+                            else:
+                                self._joined(result)
+                                return
+            elif server == 'lan':
+                self.logBox.log('Asking local network for other games...')
+                games = defer.waitForDeferred(lobby.getMulticastGames())
+                yield games
+                games = games.getResult()
                 for game in games:
                     try:
                         self.logBox.log('Found game: joining.')
@@ -141,125 +201,110 @@ class PlayAuthScreen(framework.CompoundElement):
                         pass
                     except amp.UnknownRemoteError:
                         self.logBox.log('Error on remote server.')
-                    except amp.RemoteAmpError:
+                    except amp.RemoteAmpError, e:
                         self.logBox.log('Error on remote server.')
-                        logging.writeException()
-                    except IOError:
+                        log.exception(str(e))
+                    except IOError, e:
                         self.logBox.log('Error connecting to remote server.')
-                        logging.writeException()
+                        log.exception(str(e))
                     except ConnectionFailed:
                         self.logBox.log('Could not connect.')
-                        logging.writeException()
                     else:
                         self._joined(result)
                         return
+            elif server == 'create':
+                for server in servers:
+                    if server in badServers or not isinstance(server, tuple):
+                        continue
+                    self.logBox.log('Asking to create game on %s...' %
+                            (server[0],))
+                    try:
+                       result = defer.waitForDeferred(lobby.startGame(server,
+                               self._makePasswordGetter()))
+                       yield result
+                       result = result.getResult()
+                    except ConnectError:
+                        self.logBox.log('Unable to connect.')
+                        badServers.add(server)
+                    except authcommands.CannotCreateGame:
+                        self.logBox.log('Server will not create game.')
+                    except IncorrectServerVersion:
+                        self.logBox.log('Wrong server version to create games.')
+                    except AuthenticationCancelled:
+                        pass
+                    except authcommands.NotAuthenticated:
+                        self.logBox.log('Authentication failure.')
+                    except authcommands.GameDoesNotExist:
+                        self.logBox.log(
+                                'Error connecting to newly-created game.')
+                    except amp.UnknownRemoteError:
+                        self.logBox.log('Error on remote server.')
+                    except amp.RemoteAmpError, e:
+                        self.logBox.log('Error on remote server.')
+                        log.exception(str(e))
+                    except IOError, e:
+                        self.logBox.log('Error connecting to remote server.')
+                        log.exception(str(e))
+                    else:
+                        self.logBox.log('Game created and joined.')
+                        self._joined(result)
+                        return
 
-        self.logBox.log('Asking local network for other games...')
-        games = defer.waitForDeferred(lobby.getMulticastGames())
-        yield games
-        games = games.getResult()
-        for game in games:
-            try:
-                self.logBox.log('Found game: joining.')
-                result = defer.waitForDeferred(game.join(
-                        self._makePasswordGetter()))
+        if canHost:
+            if not self.cancelled:
+                result = defer.waitForDeferred(HostGameQuery(self.app).run())
                 yield result
                 result = result.getResult()
-            except authcommands.GameDoesNotExist:
-                pass
-            except amp.UnknownRemoteError:
-                self.logBox.log('Error on remote server.')
-            except amp.RemoteAmpError:
-                self.logBox.log('Error on remote server.')
-                logging.writeException()
-            except IOError:
-                self.logBox.log('Error connecting to remote server.')
-                logging.writeException()
-            except ConnectionFailed:
-                self.logBox.log('Could not connect.')
-                logging.writeException()
-            else:
-                self._joined(result)
-                return
 
-        for server in list(servers):
-            self.logBox.log('Asking to create game on %s...' % (server[0],))
-            try:
-               result = defer.waitForDeferred(lobby.startGame(server,
-                       self._makePasswordGetter()))
-               yield result
-               result = result.getResult()
-            except ConnectError:
-                self.logBox.log('Unable to connect.')
-                servers.remove(server)
-            except authcommands.CannotCreateGame:
-                pass
-            except IncorrectServerVersion:
-                self.logBox.log('Wrong server version to create games.')
-            except AuthenticationCancelled:
-                pass
-            except authcommands.NotAuthenticated:
-                self.logBox.log('Authentication failure.')
-            except authcommands.GameDoesNotExist:
-                self.logBox.log('Error connecting to newly-created game.')
-            except amp.UnknownRemoteError:
-                self.logBox.log('Error on remote server.')
-            except amp.RemoteAmpError:
-                self.logBox.log('Error on remote server.')
-                logging.writeException()
-            except IOError:
-                self.logBox.log('Error connecting to remote server.')
-                logging.writeException()
-            else:
-                self.logBox.log('Game created and joined.')
-                self._joined(result)
-                return
+                if not result:
+                    self.onFail.execute()
+                    return
 
-        result = defer.waitForDeferred(HostGameQuery(self.app).run())
-        yield result
-        result = result.getResult()
+                nick = self.app.identitySettings.nick
+                if nick is not None:
+                    serverName = "%s's game" % (nick,)
+                else:
+                    serverName = 'A Trosnoth game'
+                self.app.startServer(serverName, 2, 1)
 
-        if not result:
-            self.onClose.execute()
-            return
+                # Notify remaining auth servers of this game.
+                for server in servers:
+                    if server in badServers or not isinstance(server, tuple):
+                        continue
+                    self.logBox.log('Registering game with %s...' %
+                            (server[0],))
+                    try:
+                       result = defer.waitForDeferred(lobby.registerGame(server,
+                               self.app.server))
+                       yield result
+                       result = result.getResult()
+                    except ConnectError:
+                        self.logBox.log('Unable to connect.')
+                    if not result:
+                        self.logBox.log('Registration failed.')
 
-
-        nick = self.app.identitySettings.nick
-        if nick is not None:
-            serverName = "%s's game" % (nick,)
+                self.onSucceed.execute()
+                self.app.interface.connectToLocalServer()
         else:
-            serverName = 'A Trosnoth game'
-        self.app.startServer(serverName, 2, 1)
-
-        # Notify remaining auth servers of this game.
-        for server in servers:
-            self.logBox.log('Registering game with %s...' % (server[0],))
-            try:
-               result = defer.waitForDeferred(lobby.registerGame(server,
-                       self.app.server))
-               yield result
-               result = result.getResult()
-            except ConnectError:
-                self.logBox.log('Unable to connect.')
-            if not result:
-                self.logBox.log('Registration failed.')
-
-        # TODO: Set up multicasting about the game.
-
-        self.onClose.execute()
-        self.app.interface.connectToLocalServer()
+            if not self.cancelled:
+                box = OkBox(self.app, ScaledSize(450, 150), 'Trosnoth',
+                        'Connecting unsuccessful.')
+                box.onClose.addListener(self.onFail.execute)
+                box.show()
 
     def _joined(self, result):
         netHandler, authTag = result
-        self.onClose.execute()
+        self.onSucceed.execute()
         self.app.interface.connectedToGame(netHandler, authTag)
 
     def cancel(self, element):
-        self.onClose.execute()
+        self.cancelled = True
+        self._makePasswordGetter().cancelled = True
+        self.onFail.execute()
 
     def processEvent(self, event):
-        if event.type == pygame.locals.KEYDOWN and event.key == pygame.K_ESCAPE:
-            self.onClose.execute()
+        if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
+            self.onFail.execute()
         else:
             return super(PlayAuthScreen, self).processEvent(event)
 
@@ -281,7 +326,8 @@ class LogBox(framework.Element):
             screen.blit(self.image, r.topleft)
 
     def log(self, text):
-        t = self.font.render(self.app, text, False, self.colour, (255, 255, 255))
+        t = self.font.render(self.app, text, False, self.colour,
+                (255, 255, 255))
         t.set_colorkey((255, 255, 255))
         r = self.region.getRect(self.app)
         img = pygame.Surface(r.size)
@@ -335,17 +381,17 @@ class HostGameQuery(DialogBox):
             return DialogBox.processEvent(self, event)
 
     def yesClicked(self, element=None):
-        self.Close()
+        self.close()
         self._deferred.callback(True)
 
     def noClicked(self, element=None):
-        self.Close()
+        self.close()
         self._deferred.callback(False)
 
     def run(self):
         if self.showing:
             raise PasswordGUIError('HostGameQuery already showing')
-        self.Show()
+        self.show()
         result = self._deferred = defer.Deferred()
         return result
 
@@ -362,8 +408,9 @@ class PasswordGUI(DialogBox):
         errorTextColour = app.theme.colours.errorColour
 
         self.tabContainer = TabContainer(app,
-                Region(topleft=self.Relative(0, 0), size=self.Relative(1, 0.75)),
-                font, app.theme.colours.tabContainerColour)
+                Region(topleft=self.Relative(0, 0),
+                size=self.Relative(1, 0.75)), font,
+                app.theme.colours.tabContainerColour)
         self.tabContainer.addTab(LoginTab(app))
         self.tabContainer.addTab(CreateAccountTab(app))
 
@@ -383,16 +430,18 @@ class PasswordGUI(DialogBox):
                 'Cancel', font, btnColour, highlightColour,
                 onClick=self.cancelClicked),
         ]
+        self.cancelled = False
 
     def processEvent(self, event):
-        if event.type == pygame.KEYDOWN and event.key in (pygame.K_KP_ENTER, pygame.K_RETURN):
+        if event.type == pygame.KEYDOWN and event.key in (pygame.K_KP_ENTER,
+                pygame.K_RETURN):
             self.okClicked()
             return None
         else:
             return DialogBox.processEvent(self, event)
 
     def cancelClicked(self, element=None):
-        self.Close()
+        self.close()
         self._deferred.callback(None)
 
     def okClicked(self, element=None):
@@ -417,18 +466,23 @@ class PasswordGUI(DialogBox):
             self.setErrorText('Password cannot be blank!')
             return
 
-        self.Close()
+        self.close()
         self.app.identitySettings.usernames[self._host] = username
         self._deferred.callback((create, username, password))
 
     def getPassword(self, host, errorMsg=''):
         if self.showing:
             raise PasswordGUIError('PasswordGUI already showing')
+        if self.cancelled:
+            self.cancelled = False
+            result = self._deferred = defer.Deferred()
+            WeakCallLater(0.1, result, 'callback', None)
+            return result
         self.setCaption(host)
         self.tabContainer.tabs[0].reset(host)
         self.tabContainer.tabs[1].reset()
         self.setErrorText(errorMsg)
-        self.Show()
+        self.show()
         self._host = host
         result = self._deferred = defer.Deferred()
         return result
