@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2010 Joshua D Bartlett
+# Copyright (C) 2006-2011 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,17 +16,19 @@
 # 02110-1301, USA.
 
 class Event(object):
-    def __init__(self):
+    def __init__(self, listener=None):
         self.listeners = set()
-        
+        if listener is not None:
+            self.addListener(listener)
+
     def addListener(self, obj):
         self.listeners.add(obj)
 
     def removeListener(self, obj):
         self.listeners.remove(obj)
-        
+
     def execute(self, *args, **kwargs):
         for call in self.listeners:
             call(*args, **kwargs)
-            
+
     __call__ = execute
