@@ -1,5 +1,5 @@
 # Trosnoth (UberTweak Platform Game)
-# Copyright (C) 2006-2010 Joshua D Bartlett
+# Copyright (C) 2006-2011 Joshua D Bartlett
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,4 +16,14 @@
 # 02110-1301, USA.
 
 def distance(pt1, pt2):
-    return ((pt1[0] - pt2[0]) ** 2 + (pt1[1] - pt2[1]) ** 2) ** 0.5
\ No newline at end of file
+    return ((pt1[0] - pt2[0]) ** 2 + (pt1[1] - pt2[1]) ** 2) ** 0.5
+
+def fadeValues(val1, val2, interval):
+    return val2 * interval + val1 * (1 - interval)
+
+def fadeColours(c1, c2, interval):
+    return (
+        fadeValues(c1[0], c2[0], interval),
+        fadeValues(c1[1], c2[1], interval),
+        fadeValues(c1[2], c2[2], interval),
+    )
